@@ -268,3 +268,109 @@ def beam_from_fwhm(fwhm_arcminute, lmax):
     return ell, bl
 
 
+def create_arbitrary_binning_file(delta_l_list, l_bound_list, binning_file=None):
+    """
+    This function return a binning scheme with non constant bin size
+    you specify a list of bin size: delta_l_list, and the corresponding bound at
+    which this bin size ceases to apply: l_bound_list
+    optionnaly write to disk a binning file in format, lmin, lmax, lmean
+
+    Parameters
+    ----------
+    delta_l_list: list of integers
+        the different binsize we want to consider
+    l_bound_list: list of integers
+        the multipole at which the binsize ceases to apply
+    write_binning_file: string
+        if you want to write to disk, pass a string with the name of the file
+    """
+ 
+    lmin_list = []
+    lmax_list = []
+    lmean_list = []
+    
+    lmin = 2
+    for count, ells in enumerate(l_bound_list):
+        while True:
+            lmax = lmin +  delta_l_list[count]
+
+            if lmax > l_bound_list[count]:
+                break
+
+            lmean = (lmax + lmin) / 2
+
+            lmin_list += [lmin]
+            lmax_list += [lmax]
+            lmean_list += [lmean]
+            
+            lmin = lmax + 1
+    if binning_file is not None:
+        np.savetxt(binning_file, np.transpose([lmin_list, lmax_list, lmean_list]))
+    
+    return lmin_list, lmax_list, lmean_list
+
+def maximum_likelihood_combination(cov_mat, P_mat, data_vec, test_matrix=False):
+    """
+    This function solve for the maximum likelihood data_vec and covariance for a problem of the form
+    chi2 = (data_vec - P ML_data_vec).T cov_mat ** -1 data_vec - P ML_data_vec)
+
+
+    Parameters
+    ----------
+    cov_mat: 2d array
+        the covariance matrix of data_vec, of size N_ini x N_ini
+    P_mat: 2d array
+        the "pointing matrix" that project the final data vector of size N_f into the initial data vector space N_ini
+        shape is (N_ini, N_f)
+    data_vec: 1d array
+        the initial data vector
+    """
+    
+    inv_cov_mat = np.linalg.inv(cov_mat)
+
+    ML_cov_mat = np.linalg.inv(np.dot(np.dot(P_mat, inv_cov_mat), P_mat.T))
+    ML_data_vec = np.dot(ML_cov_mat, np.dot(P_mat, np.dot(inv_cov_mat, data_vec)))
+    
+    if test_matrix:
+        is_symmetric(mat, tol=1e-7)
+        is_pos_def(mat)
+
+    return ML_cov_mat, ML_data_vec
+
+
+def is_symmetric(mat, tol=1e-8):
+    """
+    This function check if a matrix is symmetric
+    
+    Parameters
+    ----------
+    mat: 2d array
+        the matrix tested
+    tol:
+        the absolute tolerance on assymetry
+    """
+    if np.all(np.abs(mat-mat.T) < tol):
+        print("the tested matrix is symmetric")
+    else:
+        print("the tested matrix is not symmetric")
+    return
+
+
+def is_pos_def(mat):
+    """
+    This function check if a matrix is positive definite
+    
+    Parameters
+    ----------
+    mat: 2d array
+        the matrix tested
+    """
+    if np.all(np.linalg.eigvals(mat) > 0):
+        print("the tested matrix is positive definite")
+    else:
+        print("the tested matrix is not positive definite")
+        print(np.linalg.eigvals(mat))
+    return
+
+
+    
